# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .anchorsite_override import AnchorsiteOverride

__all__ = ["ConnectionRetrieveResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the specific resource."""

    active: Optional[bool] = None
    """Defaults to true"""

    anchorsite_override: Optional[AnchorsiteOverride] = None
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    connection_name: Optional[str] = None

    created_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    outbound_voice_profile_id: Optional[str] = None
    """Identifies the associated outbound voice profile."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    tags: Optional[List[str]] = None
    """Tags associated with the connection."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date indicating when the resource was updated."""

    webhook_api_version: Optional[Literal["1", "2"]] = None
    """Determines which webhook format will be used, Telnyx API v1 or v2."""

    webhook_event_failover_url: Optional[str] = None
    """
    The failover URL where webhooks related to this connection will be sent if
    sending to the primary URL fails.
    """

    webhook_event_url: Optional[str] = None
    """The URL where webhooks related to this connection will be sent."""


class ConnectionRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TransportProtocol"]

TransportProtocol: TypeAlias = Literal["UDP", "TCP", "TLS"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["RequirementGroupListParams", "Filter"]


class RequirementGroupListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[country_code], filter[phone_number_type], filter[action],
    filter[status], filter[customer_reference]
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[country_code], filter[phone_number_type], filter[action], filter[status], filter[customer_reference]
    """

    action: Literal["ordering", "porting", "action"]
    """Filter requirement groups by action type"""

    country_code: str
    """Filter requirement groups by country code (iso alpha 2)"""

    customer_reference: str
    """Filter requirement groups by customer reference"""

    phone_number_type: Literal["local", "toll_free", "mobile", "national", "shared_cost"]
    """Filter requirement groups by phone number type."""

    status: Literal["approved", "unapproved", "pending-approval", "declined", "expired"]
    """Filter requirement groups by status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .inbound_message import InboundMessage

__all__ = ["InboundMessageWebhookEvent"]


class InboundMessageWebhookEvent(BaseModel):
    data: Optional[InboundMessage] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    RoomRecordingListResponse,
    RoomRecordingRetrieveResponse,
    RoomRecordingDeleteBulkResponse,
)
from telnyx._utils import parse_date
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRoomRecordings:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        room_recording = client.room_recordings.retrieve(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(RoomRecordingRetrieveResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.room_recordings.with_raw_response.retrieve(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = response.parse()
        assert_matches_type(RoomRecordingRetrieveResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.room_recordings.with_streaming_response.retrieve(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = response.parse()
            assert_matches_type(RoomRecordingRetrieveResponse, room_recording, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_recording_id` but received ''"):
            client.room_recordings.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        room_recording = client.room_recordings.list()
        assert_matches_type(SyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        room_recording = client.room_recordings.list(
            filter={
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_started_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "duration_secs": 20,
                "participant_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "room_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "session_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "status": "completed",
                "type": "audio",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.room_recordings.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.room_recordings.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        room_recording = client.room_recordings.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert room_recording is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.room_recordings.with_raw_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = response.parse()
        assert room_recording is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.room_recordings.with_streaming_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = response.parse()
            assert room_recording is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_recording_id` but received ''"):
            client.room_recordings.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete_bulk(self, client: Telnyx) -> None:
        room_recording = client.room_recordings.delete_bulk()
        assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete_bulk_with_all_params(self, client: Telnyx) -> None:
        room_recording = client.room_recordings.delete_bulk(
            filter={
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_started_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "duration_secs": 20,
                "participant_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "room_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "session_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "status": "completed",
                "type": "audio",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete_bulk(self, client: Telnyx) -> None:
        response = client.room_recordings.with_raw_response.delete_bulk()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = response.parse()
        assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete_bulk(self, client: Telnyx) -> None:
        with client.room_recordings.with_streaming_response.delete_bulk() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = response.parse()
            assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncRoomRecordings:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        room_recording = await async_client.room_recordings.retrieve(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert_matches_type(RoomRecordingRetrieveResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.room_recordings.with_raw_response.retrieve(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = await response.parse()
        assert_matches_type(RoomRecordingRetrieveResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.room_recordings.with_streaming_response.retrieve(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = await response.parse()
            assert_matches_type(RoomRecordingRetrieveResponse, room_recording, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_recording_id` but received ''"):
            await async_client.room_recordings.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        room_recording = await async_client.room_recordings.list()
        assert_matches_type(AsyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        room_recording = await async_client.room_recordings.list(
            filter={
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_started_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "duration_secs": 20,
                "participant_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "room_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "session_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "status": "completed",
                "type": "audio",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.room_recordings.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.room_recordings.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[RoomRecordingListResponse], room_recording, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        room_recording = await async_client.room_recordings.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )
        assert room_recording is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.room_recordings.with_raw_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = await response.parse()
        assert room_recording is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.room_recordings.with_streaming_response.delete(
            "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = await response.parse()
            assert room_recording is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `room_recording_id` but received ''"):
            await async_client.room_recordings.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete_bulk(self, async_client: AsyncTelnyx) -> None:
        room_recording = await async_client.room_recordings.delete_bulk()
        assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete_bulk_with_all_params(self, async_client: AsyncTelnyx) -> None:
        room_recording = await async_client.room_recordings.delete_bulk(
            filter={
                "date_ended_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "date_started_at": {
                    "eq": parse_date("2021-04-25"),
                    "gte": parse_date("2021-04-25"),
                    "lte": parse_date("2021-04-25"),
                },
                "duration_secs": 20,
                "participant_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "room_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "session_id": "0ccc7b54-4df3-4bca-a65a-3da1ecc777f0",
                "status": "completed",
                "type": "audio",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete_bulk(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.room_recordings.with_raw_response.delete_bulk()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        room_recording = await response.parse()
        assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete_bulk(self, async_client: AsyncTelnyx) -> None:
        async with async_client.room_recordings.with_streaming_response.delete_bulk() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            room_recording = await response.parse()
            assert_matches_type(RoomRecordingDeleteBulkResponse, room_recording, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .export_portouts_csv_report_param import ExportPortoutsCsvReportParam

__all__ = ["ReportCreateParams"]


class ReportCreateParams(TypedDict, total=False):
    params: Required[ExportPortoutsCsvReportParam]
    """The parameters for generating a port-outs CSV report."""

    report_type: Required[Literal["export_portouts_csv"]]
    """Identifies the type of report"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .standard_pagination_meta import StandardPaginationMeta
from .telco_data_usage_report_response import TelcoDataUsageReportResponse

__all__ = ["NumberLookupListResponse"]


class NumberLookupListResponse(BaseModel):
    data: Optional[List[TelcoDataUsageReportResponse]] = None

    meta: Optional[StandardPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .telco_data_aggregation import TelcoDataAggregation

__all__ = ["TelcoDataUsageRecord"]


class TelcoDataUsageRecord(BaseModel):
    aggregations: Optional[List[TelcoDataAggregation]] = None
    """List of aggregations by lookup type"""

    record_type: Optional[str] = None
    """Record type identifier"""

    user_id: Optional[str] = None
    """User ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .batch_csv_pagination_meta import BatchCsvPaginationMeta
from .cdr_detailed_req_response import CdrDetailedReqResponse

__all__ = ["VoiceListResponse"]


class VoiceListResponse(BaseModel):
    data: Optional[List[CdrDetailedReqResponse]] = None

    meta: Optional[BatchCsvPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .integration_connection import IntegrationConnection

__all__ = ["ConnectionRetrieveResponse"]


class ConnectionRetrieveResponse(BaseModel):
    data: IntegrationConnection



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ....._models import BaseModel

__all__ = ["TelnyxAgentData"]


class TelnyxAgentData(BaseModel):
    created_at: datetime

    run_id: str

    telnyx_agent_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ResembleVoiceSettings"]


class ResembleVoiceSettings(TypedDict, total=False):
    type: Required[Literal["resemble"]]
    """Voice settings provider type"""

    format: Literal["wav", "mp3"]
    """Output audio format."""

    precision: Literal["PCM_16", "PCM_24", "PCM_32", "MULAW"]
    """Audio precision format."""

    sample_rate: Literal["8000", "16000", "22050", "32000", "44100", "48000"]
    """Audio sample rate in Hz."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["AudioTranscribeResponse", "Segment"]


class Segment(BaseModel):
    id: float
    """Unique identifier of the segment."""

    end: float
    """End time of the segment in seconds."""

    start: float
    """Start time of the segment in seconds."""

    text: str
    """Text content of the segment."""


class AudioTranscribeResponse(BaseModel):
    text: str
    """The transcribed text for the audio file."""

    duration: Optional[float] = None
    """The duration of the audio file in seconds.

    This is only included if `response_format` is set to `verbose_json`.
    """

    segments: Optional[List[Segment]] = None
    """Segments of the transcribed text and their corresponding details.

    This is only included if `response_format` is set to `verbose_json`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .buckets import (
    BucketsResource,
    AsyncBucketsResource,
    BucketsResourceWithRawResponse,
    AsyncBucketsResourceWithRawResponse,
    BucketsResourceWithStreamingResponse,
    AsyncBucketsResourceWithStreamingResponse,
)
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)

__all__ = [
    "BucketsResource",
    "AsyncBucketsResource",
    "BucketsResourceWithRawResponse",
    "AsyncBucketsResourceWithRawResponse",
    "BucketsResourceWithStreamingResponse",
    "AsyncBucketsResourceWithStreamingResponse",
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PartnerCampaignListSharedByMeResponse"]


class PartnerCampaignListSharedByMeResponse(BaseModel):
    """
    Campaign is generated by the 10DLC registry once the corresponding campaign request is approved. Each campaign is assigned a unique identifier - **campaignId**. Once a campaign is activated, limited information is published to the NetNumber OSR service for consumption by members of the ecosystem. When a campaign is suspended(reversible) or expired(non-reversible), campaign data is deleted from the OSR service. Most attributes of campaignare immutable, including **usecase**, **vertical**, **brandId** and **cspId**.
    """

    brand_id: str = FieldInfo(alias="brandId")
    """Alphanumeric identifier of the brand associated with this campaign."""

    campaign_id: str = FieldInfo(alias="campaignId")
    """Alphanumeric identifier assigned by the registry for a campaign.

    This identifier is required by the NetNumber OSR SMS enabling process of 10DLC.
    """

    usecase: str
    """Campaign usecase.

    Must be of defined valid types. Use `/10dlc/enum/usecase` operation to retrieve
    usecases available for given brand.
    """

    create_date: Optional[str] = FieldInfo(alias="createDate", default=None)
    """Unix timestamp when campaign was created."""

    status: Optional[str] = None
    """Current campaign status.

    Possible values: ACTIVE, EXPIRED. A newly created campaign defaults to ACTIVE
    status.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.assistants import version_update_params, version_retrieve_params
from ....types.ai.assistants_list import AssistantsList
from ....types.ai.enabled_features import EnabledFeatures
from ....types.ai.inference_embedding import InferenceEmbedding
from ....types.ai.assistant_tool_param import AssistantToolParam
from ....types.ai.voice_settings_param import VoiceSettingsParam
from ....types.ai.widget_settings_param import WidgetSettingsParam
from ....types.ai.insight_settings_param import InsightSettingsParam
from ....types.ai.privacy_settings_param import PrivacySettingsParam
from ....types.ai.messaging_settings_param import MessagingSettingsParam
from ....types.ai.telephony_settings_param import TelephonySettingsParam
from ....types.ai.transcription_settings_param import TranscriptionSettingsParam

__all__ = ["VersionsResource", "AsyncVersionsResource"]


class VersionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VersionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        version_id: str,
        *,
        assistant_id: str,
        include_mcp_servers: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceEmbedding:
        """
        Retrieves a specific version of an assistant by assistant_id and version_id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._get(
            f"/ai/assistants/{assistant_id}/versions/{version_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"include_mcp_servers": include_mcp_servers}, version_retrieve_params.VersionRetrieveParams
                ),
            ),
            cast_to=InferenceEmbedding,
        )

    def update(
        self,
        version_id: str,
        *,
        assistant_id: str,
        description: str | Omit = omit,
        dynamic_variables: Dict[str, object] | Omit = omit,
        dynamic_variables_webhook_url: str | Omit = omit,
        enabled_features: List[EnabledFeatures] | Omit = omit,
        greeting: str | Omit = omit,
        insight_settings: InsightSettingsParam | Omit = omit,
        instructions: str | Omit = omit,
        llm_api_key_ref: str | Omit = omit,
        messaging_settings: MessagingSettingsParam | Omit = omit,
        model: str | Omit = omit,
        name: str | Omit = omit,
        privacy_settings: PrivacySettingsParam | Omit = omit,
        telephony_settings: TelephonySettingsParam | Omit = omit,
        tools: Iterable[AssistantToolParam] | Omit = omit,
        transcription: TranscriptionSettingsParam | Omit = omit,
        voice_settings: VoiceSettingsParam | Omit = omit,
        widget_settings: WidgetSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceEmbedding:
        """Updates the configuration of a specific assistant version.

        Can not update main
        version

        Args:
          dynamic_variables: Map of dynamic variables and their default values

          dynamic_variables_webhook_url: If the dynamic_variables_webhook_url is set for the assistant, we will send a
              request at the start of the conversation. See our
              [guide](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)
              for more information.

          greeting: Text that the assistant will use to start the conversation. This may be
              templated with
              [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables).
              Use an empty string to have the assistant wait for the user to speak first. Use
              the special value `<assistant-speaks-first-with-model-generated-message>` to
              have the assistant generate the greeting based on the system instructions.

          instructions: System instructions for the assistant. These may be templated with
              [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)

          llm_api_key_ref: This is only needed when using third-party inference providers. The `identifier`
              for an integration secret
              [/v2/integration_secrets](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
              that refers to your LLM provider's API key. Warning: Free plans are unlikely to
              work with this integration.

          model: ID of the model to use. You can use the
              [Get models API](https://developers.telnyx.com/api-reference/chat/get-available-models)
              to see all of your available models,

          tools: The tools that the assistant can use. These may be templated with
              [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)

          widget_settings: Configuration settings for the assistant's web widget.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._post(
            f"/ai/assistants/{assistant_id}/versions/{version_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "dynamic_variables": dynamic_variables,
                    "dynamic_variables_webhook_url": dynamic_variables_webhook_url,
                    "enabled_features": enabled_features,
                    "greeting": greeting,
                    "insight_settings": insight_settings,
                    "instructions": instructions,
                    "llm_api_key_ref": llm_api_key_ref,
                    "messaging_settings": messaging_settings,
                    "model": model,
                    "name": name,
                    "privacy_settings": privacy_settings,
                    "telephony_settings": telephony_settings,
                    "tools": tools,
                    "transcription": transcription,
                    "voice_settings": voice_settings,
                    "widget_settings": widget_settings,
                },
                version_update_params.VersionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InferenceEmbedding,
        )

    def list(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantsList:
        """
        Retrieves all versions of a specific assistant with complete configuration and
        metadata

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return self._get(
            f"/ai/assistants/{assistant_id}/versions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantsList,
        )

    def delete(
        self,
        version_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently removes a specific version of an assistant.

        Can not delete main
        version

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/assistants/{assistant_id}/versions/{version_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def promote(
        self,
        version_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceEmbedding:
        """
        Promotes a specific version to be the main/current version of the assistant.
        This will delete any existing canary deploy configuration and send all live
        production traffic to this version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return self._post(
            f"/ai/assistants/{assistant_id}/versions/{version_id}/promote",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InferenceEmbedding,
        )


class AsyncVersionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVersionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVersionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVersionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVersionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        version_id: str,
        *,
        assistant_id: str,
        include_mcp_servers: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceEmbedding:
        """
        Retrieves a specific version of an assistant by assistant_id and version_id

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._get(
            f"/ai/assistants/{assistant_id}/versions/{version_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"include_mcp_servers": include_mcp_servers}, version_retrieve_params.VersionRetrieveParams
                ),
            ),
            cast_to=InferenceEmbedding,
        )

    async def update(
        self,
        version_id: str,
        *,
        assistant_id: str,
        description: str | Omit = omit,
        dynamic_variables: Dict[str, object] | Omit = omit,
        dynamic_variables_webhook_url: str | Omit = omit,
        enabled_features: List[EnabledFeatures] | Omit = omit,
        greeting: str | Omit = omit,
        insight_settings: InsightSettingsParam | Omit = omit,
        instructions: str | Omit = omit,
        llm_api_key_ref: str | Omit = omit,
        messaging_settings: MessagingSettingsParam | Omit = omit,
        model: str | Omit = omit,
        name: str | Omit = omit,
        privacy_settings: PrivacySettingsParam | Omit = omit,
        telephony_settings: TelephonySettingsParam | Omit = omit,
        tools: Iterable[AssistantToolParam] | Omit = omit,
        transcription: TranscriptionSettingsParam | Omit = omit,
        voice_settings: VoiceSettingsParam | Omit = omit,
        widget_settings: WidgetSettingsParam | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceEmbedding:
        """Updates the configuration of a specific assistant version.

        Can not update main
        version

        Args:
          dynamic_variables: Map of dynamic variables and their default values

          dynamic_variables_webhook_url: If the dynamic_variables_webhook_url is set for the assistant, we will send a
              request at the start of the conversation. See our
              [guide](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)
              for more information.

          greeting: Text that the assistant will use to start the conversation. This may be
              templated with
              [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables).
              Use an empty string to have the assistant wait for the user to speak first. Use
              the special value `<assistant-speaks-first-with-model-generated-message>` to
              have the assistant generate the greeting based on the system instructions.

          instructions: System instructions for the assistant. These may be templated with
              [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)

          llm_api_key_ref: This is only needed when using third-party inference providers. The `identifier`
              for an integration secret
              [/v2/integration_secrets](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
              that refers to your LLM provider's API key. Warning: Free plans are unlikely to
              work with this integration.

          model: ID of the model to use. You can use the
              [Get models API](https://developers.telnyx.com/api-reference/chat/get-available-models)
              to see all of your available models,

          tools: The tools that the assistant can use. These may be templated with
              [dynamic variables](https://developers.telnyx.com/docs/inference/ai-assistants/dynamic-variables)

          widget_settings: Configuration settings for the assistant's web widget.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._post(
            f"/ai/assistants/{assistant_id}/versions/{version_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "dynamic_variables": dynamic_variables,
                    "dynamic_variables_webhook_url": dynamic_variables_webhook_url,
                    "enabled_features": enabled_features,
                    "greeting": greeting,
                    "insight_settings": insight_settings,
                    "instructions": instructions,
                    "llm_api_key_ref": llm_api_key_ref,
                    "messaging_settings": messaging_settings,
                    "model": model,
                    "name": name,
                    "privacy_settings": privacy_settings,
                    "telephony_settings": telephony_settings,
                    "tools": tools,
                    "transcription": transcription,
                    "voice_settings": voice_settings,
                    "widget_settings": widget_settings,
                },
                version_update_params.VersionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InferenceEmbedding,
        )

    async def list(
        self,
        assistant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AssistantsList:
        """
        Retrieves all versions of a specific assistant with complete configuration and
        metadata

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        return await self._get(
            f"/ai/assistants/{assistant_id}/versions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AssistantsList,
        )

    async def delete(
        self,
        version_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Permanently removes a specific version of an assistant.

        Can not delete main
        version

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/assistants/{assistant_id}/versions/{version_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def promote(
        self,
        version_id: str,
        *,
        assistant_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InferenceEmbedding:
        """
        Promotes a specific version to be the main/current version of the assistant.
        This will delete any existing canary deploy configuration and send all live
        production traffic to this version.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not assistant_id:
            raise ValueError(f"Expected a non-empty value for `assistant_id` but received {assistant_id!r}")
        if not version_id:
            raise ValueError(f"Expected a non-empty value for `version_id` but received {version_id!r}")
        return await self._post(
            f"/ai/assistants/{assistant_id}/versions/{version_id}/promote",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InferenceEmbedding,
        )


class VersionsResourceWithRawResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.retrieve = to_raw_response_wrapper(
            versions.retrieve,
        )
        self.update = to_raw_response_wrapper(
            versions.update,
        )
        self.list = to_raw_response_wrapper(
            versions.list,
        )
        self.delete = to_raw_response_wrapper(
            versions.delete,
        )
        self.promote = to_raw_response_wrapper(
            versions.promote,
        )


class AsyncVersionsResourceWithRawResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.retrieve = async_to_raw_response_wrapper(
            versions.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            versions.update,
        )
        self.list = async_to_raw_response_wrapper(
            versions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            versions.delete,
        )
        self.promote = async_to_raw_response_wrapper(
            versions.promote,
        )


class VersionsResourceWithStreamingResponse:
    def __init__(self, versions: VersionsResource) -> None:
        self._versions = versions

        self.retrieve = to_streamed_response_wrapper(
            versions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            versions.update,
        )
        self.list = to_streamed_response_wrapper(
            versions.list,
        )
        self.delete = to_streamed_response_wrapper(
            versions.delete,
        )
        self.promote = to_streamed_response_wrapper(
            versions.promote,
        )


class AsyncVersionsResourceWithStreamingResponse:
    def __init__(self, versions: AsyncVersionsResource) -> None:
        self._versions = versions

        self.retrieve = async_to_streamed_response_wrapper(
            versions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            versions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            versions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            versions.delete,
        )
        self.promote = async_to_streamed_response_wrapper(
            versions.promote,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .outbound_message_payload import OutboundMessagePayload

__all__ = ["MessageRetrieveGroupMessagesResponse"]


class MessageRetrieveGroupMessagesResponse(BaseModel):
    data: Optional[List[OutboundMessagePayload]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = [
    "RegulatoryRequirementRetrieveResponse",
    "Data",
    "DataRegulatoryRequirement",
    "DataRegulatoryRequirementAcceptanceCriteria",
]


class DataRegulatoryRequirementAcceptanceCriteria(BaseModel):
    acceptable_characters: Optional[str] = None

    acceptable_values: Optional[List[str]] = None

    case_sensitive: Optional[str] = None

    locality_limit: Optional[str] = None

    max_length: Optional[str] = None

    min_length: Optional[str] = None

    regex: Optional[str] = None

    time_limit: Optional[str] = None


class DataRegulatoryRequirement(BaseModel):
    id: Optional[str] = None

    acceptance_criteria: Optional[DataRegulatoryRequirementAcceptanceCriteria] = None

    description: Optional[str] = None

    example: Optional[str] = None

    field_type: Optional[str] = None

    name: Optional[str] = None


class Data(BaseModel):
    action: Optional[str] = None

    country_code: Optional[str] = None

    phone_number_type: Optional[str] = None

    regulatory_requirements: Optional[List[DataRegulatoryRequirement]] = None


class RegulatoryRequirementRetrieveResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_speak_ended import ConferenceSpeakEnded

__all__ = ["ConferenceSpeakEndedWebhookEvent"]


class ConferenceSpeakEndedWebhookEvent(BaseModel):
    data: Optional[ConferenceSpeakEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["MonthDetail"]


class MonthDetail(BaseModel):
    mrc: str
    """Monthly recurring charge amount as decimal string"""

    quantity: int
    """Number of items"""

    otc: Optional[str] = None
    """One-time charge amount as decimal string"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .user_bundles import (
    UserBundlesResource,
    AsyncUserBundlesResource,
    UserBundlesResourceWithRawResponse,
    AsyncUserBundlesResourceWithRawResponse,
    UserBundlesResourceWithStreamingResponse,
    AsyncUserBundlesResourceWithStreamingResponse,
)
from .billing_bundles import (
    BillingBundlesResource,
    AsyncBillingBundlesResource,
    BillingBundlesResourceWithRawResponse,
    AsyncBillingBundlesResourceWithRawResponse,
    BillingBundlesResourceWithStreamingResponse,
    AsyncBillingBundlesResourceWithStreamingResponse,
)

__all__ = ["BundlePricingResource", "AsyncBundlePricingResource"]


class BundlePricingResource(SyncAPIResource):
    @cached_property
    def billing_bundles(self) -> BillingBundlesResource:
        return BillingBundlesResource(self._client)

    @cached_property
    def user_bundles(self) -> UserBundlesResource:
        return UserBundlesResource(self._client)

    @cached_property
    def with_raw_response(self) -> BundlePricingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BundlePricingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BundlePricingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BundlePricingResourceWithStreamingResponse(self)


class AsyncBundlePricingResource(AsyncAPIResource):
    @cached_property
    def billing_bundles(self) -> AsyncBillingBundlesResource:
        return AsyncBillingBundlesResource(self._client)

    @cached_property
    def user_bundles(self) -> AsyncUserBundlesResource:
        return AsyncUserBundlesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBundlePricingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBundlePricingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBundlePricingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBundlePricingResourceWithStreamingResponse(self)


class BundlePricingResourceWithRawResponse:
    def __init__(self, bundle_pricing: BundlePricingResource) -> None:
        self._bundle_pricing = bundle_pricing

    @cached_property
    def billing_bundles(self) -> BillingBundlesResourceWithRawResponse:
        return BillingBundlesResourceWithRawResponse(self._bundle_pricing.billing_bundles)

    @cached_property
    def user_bundles(self) -> UserBundlesResourceWithRawResponse:
        return UserBundlesResourceWithRawResponse(self._bundle_pricing.user_bundles)


class AsyncBundlePricingResourceWithRawResponse:
    def __init__(self, bundle_pricing: AsyncBundlePricingResource) -> None:
        self._bundle_pricing = bundle_pricing

    @cached_property
    def billing_bundles(self) -> AsyncBillingBundlesResourceWithRawResponse:
        return AsyncBillingBundlesResourceWithRawResponse(self._bundle_pricing.billing_bundles)

    @cached_property
    def user_bundles(self) -> AsyncUserBundlesResourceWithRawResponse:
        return AsyncUserBundlesResourceWithRawResponse(self._bundle_pricing.user_bundles)


class BundlePricingResourceWithStreamingResponse:
    def __init__(self, bundle_pricing: BundlePricingResource) -> None:
        self._bundle_pricing = bundle_pricing

    @cached_property
    def billing_bundles(self) -> BillingBundlesResourceWithStreamingResponse:
        return BillingBundlesResourceWithStreamingResponse(self._bundle_pricing.billing_bundles)

    @cached_property
    def user_bundles(self) -> UserBundlesResourceWithStreamingResponse:
        return UserBundlesResourceWithStreamingResponse(self._bundle_pricing.user_bundles)


class AsyncBundlePricingResourceWithStreamingResponse:
    def __init__(self, bundle_pricing: AsyncBundlePricingResource) -> None:
        self._bundle_pricing = bundle_pricing

    @cached_property
    def billing_bundles(self) -> AsyncBillingBundlesResourceWithStreamingResponse:
        return AsyncBillingBundlesResourceWithStreamingResponse(self._bundle_pricing.billing_bundles)

    @cached_property
    def user_bundles(self) -> AsyncUserBundlesResourceWithStreamingResponse:
        return AsyncUserBundlesResourceWithStreamingResponse(self._bundle_pricing.user_bundles)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["WhatsappMediaParam"]


class WhatsappMediaParam(TypedDict, total=False):
    caption: str
    """media caption"""

    filename: str
    """file name with extension"""

    link: str
    """media URL"""

    voice: bool
    """true if voice message"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from datetime import date

from .._models import BaseModel

__all__ = ["ChargesBreakdownRetrieveResponse", "Data", "DataResult", "DataResultService"]


class DataResultService(BaseModel):
    cost: str
    """Cost per unit as decimal string"""

    cost_type: str
    """Type of cost (MRC or OTC)"""

    name: str
    """Service name"""


class DataResult(BaseModel):
    charge_type: str
    """Type of charge for the number"""

    service_owner_email: str
    """Email address of the service owner"""

    service_owner_user_id: str
    """User ID of the service owner"""

    services: List[DataResultService]
    """List of services associated with this number"""

    tn: str
    """Phone number"""


class Data(BaseModel):
    currency: str
    """Currency code"""

    end_date: date
    """End date of the breakdown period"""

    results: List[DataResult]
    """List of phone number charge breakdowns"""

    start_date: date
    """Start date of the breakdown period"""

    user_email: str
    """User email address"""

    user_id: str
    """User identifier"""


class ChargesBreakdownRetrieveResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_recording_saved import CallRecordingSaved

__all__ = ["CallRecordingSavedWebhookEvent"]


class CallRecordingSavedWebhookEvent(BaseModel):
    data: Optional[CallRecordingSaved] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_participant_speak_started import ConferenceParticipantSpeakStarted

__all__ = ["ConferenceParticipantSpeakStartedWebhookEvent"]


class ConferenceParticipantSpeakStartedWebhookEvent(BaseModel):
    data: Optional[ConferenceParticipantSpeakStarted] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PortingEventNewCommentEvent", "Payload", "PayloadComment"]


class PayloadComment(BaseModel):
    """The comment that was added to the porting order."""

    id: Optional[str] = None
    """Identifies the comment."""

    body: Optional[str] = None
    """The body of the comment."""

    inserted_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the comment was created."""

    user_id: Optional[str] = None
    """Identifies the user that create the comment."""

    user_type: Optional[Literal["user", "admin", "system"]] = None
    """Identifies the type of the user that created the comment."""


class Payload(BaseModel):
    """The webhook payload for the porting_order.new_comment event"""

    comment: Optional[PayloadComment] = None
    """The comment that was added to the porting order."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order that the comment was added to."""

    support_key: Optional[str] = None
    """Identifies the support key associated with the porting order."""


class PortingEventNewCommentEvent(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook", "webhook_v1"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[
        Literal[
            "porting_order.deleted",
            "porting_order.loa_updated",
            "porting_order.messaging_changed",
            "porting_order.status_changed",
            "porting_order.sharing_token_expired",
            "porting_order.new_comment",
            "porting_order.split",
        ]
    ] = None
    """Identifies the event type"""

    payload: Optional[Payload] = None
    """The webhook payload for the porting_order.new_comment event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import TypedDict

__all__ = ["NotificationSettingCreateParams", "Parameter"]


class NotificationSettingCreateParams(TypedDict, total=False):
    notification_channel_id: str
    """A UUID reference to the associated Notification Channel."""

    notification_event_condition_id: str
    """A UUID reference to the associated Notification Event Condition."""

    notification_profile_id: str
    """A UUID reference to the associated Notification Profile."""

    parameters: Iterable[Parameter]


class Parameter(TypedDict, total=False):
    name: str

    value: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from ..porting_orders_activation_job import PortingOrdersActivationJob

__all__ = ["ActionActivateResponse"]


class ActionActivateResponse(BaseModel):
    data: Optional[PortingOrdersActivationJob] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["TextToSpeechListVoicesResponse", "Voice"]


class Voice(BaseModel):
    """A voice available for text-to-speech synthesis."""

    gender: Optional[str] = None
    """Voice gender."""

    language: Optional[str] = None
    """Language code."""

    name: Optional[str] = None
    """Voice name."""

    provider: Optional[str] = None
    """The TTS provider."""

    voice_id: Optional[str] = None
    """Voice identifier."""


class TextToSpeechListVoicesResponse(BaseModel):
    """List of available voices."""

    voices: Optional[List[Voice]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import TextToSpeechListVoicesResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTextToSpeech:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_voices(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.list_voices()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_voices_with_all_params(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.list_voices(
            api_key="api_key",
            provider="aws",
        )
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_voices(self, client: Telnyx) -> None:
        response = client.text_to_speech.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = response.parse()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_voices(self, client: Telnyx) -> None:
        with client.text_to_speech.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = response.parse()
            assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_stream(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.stream()
        assert text_to_speech is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_stream_with_all_params(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.stream(
            audio_format="pcm",
            disable_cache=True,
            model_id="model_id",
            provider="aws",
            socket_id="socket_id",
            voice="voice",
            voice_id="voice_id",
        )
        assert text_to_speech is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_stream(self, client: Telnyx) -> None:
        response = client.text_to_speech.with_raw_response.stream()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = response.parse()
        assert text_to_speech is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_stream(self, client: Telnyx) -> None:
        with client.text_to_speech.with_streaming_response.stream() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = response.parse()
            assert text_to_speech is None

        assert cast(Any, response.is_closed) is True


class TestAsyncTextToSpeech:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_voices(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.list_voices()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_voices_with_all_params(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.list_voices(
            api_key="api_key",
            provider="aws",
        )
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_voices(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.text_to_speech.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = await response.parse()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_voices(self, async_client: AsyncTelnyx) -> None:
        async with async_client.text_to_speech.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = await response.parse()
            assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_stream(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.stream()
        assert text_to_speech is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_stream_with_all_params(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.stream(
            audio_format="pcm",
            disable_cache=True,
            model_id="model_id",
            provider="aws",
            socket_id="socket_id",
            voice="voice",
            voice_id="voice_id",
        )
        assert text_to_speech is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_stream(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.text_to_speech.with_raw_response.stream()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = await response.parse()
        assert text_to_speech is None

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_stream(self, async_client: AsyncTelnyx) -> None:
        async with async_client.text_to_speech.with_streaming_response.stream() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = await response.parse()
            assert text_to_speech is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import (
    authentication_provider_list_params,
    authentication_provider_create_params,
    authentication_provider_update_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.settings_param import SettingsParam
from ..types.authentication_provider import AuthenticationProvider
from ..types.authentication_provider_create_response import AuthenticationProviderCreateResponse
from ..types.authentication_provider_delete_response import AuthenticationProviderDeleteResponse
from ..types.authentication_provider_update_response import AuthenticationProviderUpdateResponse
from ..types.authentication_provider_retrieve_response import AuthenticationProviderRetrieveResponse

__all__ = ["AuthenticationProvidersResource", "AsyncAuthenticationProvidersResource"]


class AuthenticationProvidersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthenticationProvidersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AuthenticationProvidersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthenticationProvidersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AuthenticationProvidersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        settings: SettingsParam,
        short_name: str,
        active: bool | Omit = omit,
        settings_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderCreateResponse:
        """
        Creates an authentication provider.

        Args:
          name: The name associated with the authentication provider.

          settings: The settings associated with the authentication provider.

          short_name: The short name associated with the authentication provider. This must be unique
              and URL-friendly, as it's going to be part of the login URL.

          active: The active status of the authentication provider

          settings_url: The URL for the identity provider metadata file to populate the settings
              automatically. If the settings attribute is provided, that will be used instead.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/authentication_providers",
            body=maybe_transform(
                {
                    "name": name,
                    "settings": settings,
                    "short_name": short_name,
                    "active": active,
                    "settings_url": settings_url,
                },
                authentication_provider_create_params.AuthenticationProviderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderRetrieveResponse:
        """
        Retrieves the details of an existing authentication provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/authentication_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        name: str | Omit = omit,
        settings: SettingsParam | Omit = omit,
        settings_url: str | Omit = omit,
        short_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderUpdateResponse:
        """
        Updates settings of an existing authentication provider.

        Args:
          active: The active status of the authentication provider

          name: The name associated with the authentication provider.

          settings: The settings associated with the authentication provider.

          settings_url: The URL for the identity provider metadata file to populate the settings
              automatically. If the settings attribute is provided, that will be used instead.

          short_name: The short name associated with the authentication provider. This must be unique
              and URL-friendly, as it's going to be part of the login URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/authentication_providers/{id}",
            body=maybe_transform(
                {
                    "active": active,
                    "name": name,
                    "settings": settings,
                    "settings_url": settings_url,
                    "short_name": short_name,
                },
                authentication_provider_update_params.AuthenticationProviderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal[
            "name",
            "-name",
            "short_name",
            "-short_name",
            "active",
            "-active",
            "created_at",
            "-created_at",
            "updated_at",
            "-updated_at",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[AuthenticationProvider]:
        """
        Returns a list of your SSO authentication providers.

        Args:
          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code>-</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>name</code>: sorts the result by the
                  <code>name</code> field in ascending order.
                </li>
                <li>
                  <code>-name</code>: sorts the result by the
                  <code>name</code> field in descending order.
                </li>
              </ul><br/>If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/authentication_providers",
            page=SyncDefaultFlatPagination[AuthenticationProvider],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    authentication_provider_list_params.AuthenticationProviderListParams,
                ),
            ),
            model=AuthenticationProvider,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderDeleteResponse:
        """
        Deletes an existing authentication provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/authentication_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderDeleteResponse,
        )


class AsyncAuthenticationProvidersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthenticationProvidersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthenticationProvidersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthenticationProvidersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncAuthenticationProvidersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        settings: SettingsParam,
        short_name: str,
        active: bool | Omit = omit,
        settings_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderCreateResponse:
        """
        Creates an authentication provider.

        Args:
          name: The name associated with the authentication provider.

          settings: The settings associated with the authentication provider.

          short_name: The short name associated with the authentication provider. This must be unique
              and URL-friendly, as it's going to be part of the login URL.

          active: The active status of the authentication provider

          settings_url: The URL for the identity provider metadata file to populate the settings
              automatically. If the settings attribute is provided, that will be used instead.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/authentication_providers",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "settings": settings,
                    "short_name": short_name,
                    "active": active,
                    "settings_url": settings_url,
                },
                authentication_provider_create_params.AuthenticationProviderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderRetrieveResponse:
        """
        Retrieves the details of an existing authentication provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/authentication_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        active: bool | Omit = omit,
        name: str | Omit = omit,
        settings: SettingsParam | Omit = omit,
        settings_url: str | Omit = omit,
        short_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderUpdateResponse:
        """
        Updates settings of an existing authentication provider.

        Args:
          active: The active status of the authentication provider

          name: The name associated with the authentication provider.

          settings: The settings associated with the authentication provider.

          settings_url: The URL for the identity provider metadata file to populate the settings
              automatically. If the settings attribute is provided, that will be used instead.

          short_name: The short name associated with the authentication provider. This must be unique
              and URL-friendly, as it's going to be part of the login URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/authentication_providers/{id}",
            body=await async_maybe_transform(
                {
                    "active": active,
                    "name": name,
                    "settings": settings,
                    "settings_url": settings_url,
                    "short_name": short_name,
                },
                authentication_provider_update_params.AuthenticationProviderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        sort: Literal[
            "name",
            "-name",
            "short_name",
            "-short_name",
            "active",
            "-active",
            "created_at",
            "-created_at",
            "updated_at",
            "-updated_at",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AuthenticationProvider, AsyncDefaultFlatPagination[AuthenticationProvider]]:
        """
        Returns a list of your SSO authentication providers.

        Args:
          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code>-</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>name</code>: sorts the result by the
                  <code>name</code> field in ascending order.
                </li>
                <li>
                  <code>-name</code>: sorts the result by the
                  <code>name</code> field in descending order.
                </li>
              </ul><br/>If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/authentication_providers",
            page=AsyncDefaultFlatPagination[AuthenticationProvider],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                        "sort": sort,
                    },
                    authentication_provider_list_params.AuthenticationProviderListParams,
                ),
            ),
            model=AuthenticationProvider,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthenticationProviderDeleteResponse:
        """
        Deletes an existing authentication provider.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/authentication_providers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationProviderDeleteResponse,
        )


class AuthenticationProvidersResourceWithRawResponse:
    def __init__(self, authentication_providers: AuthenticationProvidersResource) -> None:
        self._authentication_providers = authentication_providers

        self.create = to_raw_response_wrapper(
            authentication_providers.create,
        )
        self.retrieve = to_raw_response_wrapper(
            authentication_providers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            authentication_providers.update,
        )
        self.list = to_raw_response_wrapper(
            authentication_providers.list,
        )
        self.delete = to_raw_response_wrapper(
            authentication_providers.delete,
        )


class AsyncAuthenticationProvidersResourceWithRawResponse:
    def __init__(self, authentication_providers: AsyncAuthenticationProvidersResource) -> None:
        self._authentication_providers = authentication_providers

        self.create = async_to_raw_response_wrapper(
            authentication_providers.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            authentication_providers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            authentication_providers.update,
        )
        self.list = async_to_raw_response_wrapper(
            authentication_providers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            authentication_providers.delete,
        )


class AuthenticationProvidersResourceWithStreamingResponse:
    def __init__(self, authentication_providers: AuthenticationProvidersResource) -> None:
        self._authentication_providers = authentication_providers

        self.create = to_streamed_response_wrapper(
            authentication_providers.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            authentication_providers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            authentication_providers.update,
        )
        self.list = to_streamed_response_wrapper(
            authentication_providers.list,
        )
        self.delete = to_streamed_response_wrapper(
            authentication_providers.delete,
        )


class AsyncAuthenticationProvidersResourceWithStreamingResponse:
    def __init__(self, authentication_providers: AsyncAuthenticationProvidersResource) -> None:
        self._authentication_providers = authentication_providers

        self.create = async_to_streamed_response_wrapper(
            authentication_providers.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            authentication_providers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            authentication_providers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            authentication_providers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            authentication_providers.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RcsContentInfoParam"]


class RcsContentInfoParam(TypedDict, total=False):
    file_url: Required[str]
    """Publicly reachable URL of the file."""

    force_refresh: bool
    """If set the URL content will not be cached."""

    thumbnail_url: str
    """Publicly reachable URL of the thumbnail. Maximum size of 100 kB."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["FaxCreateParams"]


class FaxCreateParams(TypedDict, total=False):
    connection_id: Required[str]
    """The connection ID to send the fax with."""

    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """The phone number, in E.164 format, the fax will be sent from."""

    to: Required[str]
    """The phone number, in E.164 format, the fax will be sent to or SIP URI"""

    black_threshold: int
    """The black threshold percentage for monochrome faxes.

    Only applicable if `monochrome` is set to `true`.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    from_display_name: str
    """
    The `from_display_name` string to be used as the caller id name (SIP From
    Display Name) presented to the destination (`to` number). The string should have
    a maximum of 128 characters, containing only letters, numbers, spaces, and
    -\\__~!.+ special characters. If ommited, the display name will be the same as the
    number in the `from` field.
    """

    media_name: str
    """The media_name used for the fax's media.

    Must point to a file previously uploaded to api.telnyx.com/v2/media by the same
    user/organization. media_name and media_url/contents can't be submitted
    together.
    """

    media_url: str
    """The URL (or list of URLs) to the PDF used for the fax's media.

    media_url and media_name/contents can't be submitted together.
    """

    monochrome: bool
    """The flag to enable monochrome, true black and white fax results."""

    preview_format: Literal["pdf", "tiff"]
    """The format for the preview file in case the `store_preview` is `true`."""

    quality: Literal["normal", "high", "very_high", "ultra_light", "ultra_dark"]
    """The quality of the fax.

    The `ultra` settings provides the highest quality available, but also present
    longer fax processing times. `ultra_light` is best suited for images, wihle
    `ultra_dark` is best suited for text.
    """

    store_media: bool
    """Should fax media be stored on temporary URL.

    It does not support media_name, they can't be submitted together.
    """

    store_preview: bool
    """Should fax preview be stored on temporary URL."""

    t38_enabled: bool
    """The flag to disable the T.38 protocol."""

    webhook_url: str
    """
    Use this field to override the URL to which Telnyx will send subsequent webhooks
    for this fax.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import mobile_network_operator_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.mobile_network_operator_list_response import MobileNetworkOperatorListResponse

__all__ = ["MobileNetworkOperatorsResource", "AsyncMobileNetworkOperatorsResource"]


class MobileNetworkOperatorsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MobileNetworkOperatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MobileNetworkOperatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MobileNetworkOperatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MobileNetworkOperatorsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: mobile_network_operator_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[MobileNetworkOperatorListResponse]:
        """
        Telnyx has a set of GSM mobile operators partners that are available through our
        mobile network roaming. This resource is entirely managed by Telnyx and may
        change over time. That means that this resource won't allow any write operations
        for it. Still, it's available so it can be used as a support resource that can
        be related to other resources or become a configuration option.

        Args:
          filter: Consolidated filter parameter for mobile network operators (deepObject style).
              Originally: filter[name][starts_with], filter[name][contains],
              filter[name][ends_with], filter[country_code], filter[mcc], filter[mnc],
              filter[tadig], filter[network_preferences_enabled]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/mobile_network_operators",
            page=SyncDefaultFlatPagination[MobileNetworkOperatorListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    mobile_network_operator_list_params.MobileNetworkOperatorListParams,
                ),
            ),
            model=MobileNetworkOperatorListResponse,
        )


class AsyncMobileNetworkOperatorsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMobileNetworkOperatorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMobileNetworkOperatorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMobileNetworkOperatorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMobileNetworkOperatorsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: mobile_network_operator_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        MobileNetworkOperatorListResponse, AsyncDefaultFlatPagination[MobileNetworkOperatorListResponse]
    ]:
        """
        Telnyx has a set of GSM mobile operators partners that are available through our
        mobile network roaming. This resource is entirely managed by Telnyx and may
        change over time. That means that this resource won't allow any write operations
        for it. Still, it's available so it can be used as a support resource that can
        be related to other resources or become a configuration option.

        Args:
          filter: Consolidated filter parameter for mobile network operators (deepObject style).
              Originally: filter[name][starts_with], filter[name][contains],
              filter[name][ends_with], filter[country_code], filter[mcc], filter[mnc],
              filter[tadig], filter[network_preferences_enabled]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/mobile_network_operators",
            page=AsyncDefaultFlatPagination[MobileNetworkOperatorListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    mobile_network_operator_list_params.MobileNetworkOperatorListParams,
                ),
            ),
            model=MobileNetworkOperatorListResponse,
        )


class MobileNetworkOperatorsResourceWithRawResponse:
    def __init__(self, mobile_network_operators: MobileNetworkOperatorsResource) -> None:
        self._mobile_network_operators = mobile_network_operators

        self.list = to_raw_response_wrapper(
            mobile_network_operators.list,
        )


class AsyncMobileNetworkOperatorsResourceWithRawResponse:
    def __init__(self, mobile_network_operators: AsyncMobileNetworkOperatorsResource) -> None:
        self._mobile_network_operators = mobile_network_operators

        self.list = async_to_raw_response_wrapper(
            mobile_network_operators.list,
        )


class MobileNetworkOperatorsResourceWithStreamingResponse:
    def __init__(self, mobile_network_operators: MobileNetworkOperatorsResource) -> None:
        self._mobile_network_operators = mobile_network_operators

        self.list = to_streamed_response_wrapper(
            mobile_network_operators.list,
        )


class AsyncMobileNetworkOperatorsResourceWithStreamingResponse:
    def __init__(self, mobile_network_operators: AsyncMobileNetworkOperatorsResource) -> None:
        self._mobile_network_operators = mobile_network_operators

        self.list = async_to_streamed_response_wrapper(
            mobile_network_operators.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    TelephonyCredential,
    TelephonyCredentialCreateResponse,
    TelephonyCredentialDeleteResponse,
    TelephonyCredentialUpdateResponse,
    TelephonyCredentialRetrieveResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTelephonyCredentials:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.create(
            connection_id="1234567890",
        )
        assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.create(
            connection_id="1234567890",
            expires_at="2018-02-02T22:25:27.521Z",
            name="My-new-credential",
            tag="some_tag",
        )
        assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.telephony_credentials.with_raw_response.create(
            connection_id="1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = response.parse()
        assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.telephony_credentials.with_streaming_response.create(
            connection_id="1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = response.parse()
            assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.retrieve(
            "id",
        )
        assert_matches_type(TelephonyCredentialRetrieveResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.telephony_credentials.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = response.parse()
        assert_matches_type(TelephonyCredentialRetrieveResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.telephony_credentials.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = response.parse()
            assert_matches_type(TelephonyCredentialRetrieveResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.telephony_credentials.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.update(
            id="id",
        )
        assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.update(
            id="id",
            connection_id="987654321",
            expires_at="2018-02-02T22:25:27.521Z",
            name="My-new-updated-credential",
            tag="some_tag",
        )
        assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.telephony_credentials.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = response.parse()
        assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.telephony_credentials.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = response.parse()
            assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.telephony_credentials.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.list()
        assert_matches_type(SyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.list(
            filter={
                "name": "name",
                "resource_id": "resource_id",
                "sip_username": "sip_username",
                "status": "status",
                "tag": "tag",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(SyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.telephony_credentials.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.telephony_credentials.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.delete(
            "id",
        )
        assert_matches_type(TelephonyCredentialDeleteResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.telephony_credentials.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = response.parse()
        assert_matches_type(TelephonyCredentialDeleteResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.telephony_credentials.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = response.parse()
            assert_matches_type(TelephonyCredentialDeleteResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.telephony_credentials.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_token(self, client: Telnyx) -> None:
        telephony_credential = client.telephony_credentials.create_token(
            "id",
        )
        assert_matches_type(str, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create_token(self, client: Telnyx) -> None:
        response = client.telephony_credentials.with_raw_response.create_token(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = response.parse()
        assert_matches_type(str, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create_token(self, client: Telnyx) -> None:
        with client.telephony_credentials.with_streaming_response.create_token(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = response.parse()
            assert_matches_type(str, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create_token(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.telephony_credentials.with_raw_response.create_token(
                "",
            )


class TestAsyncTelephonyCredentials:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.create(
            connection_id="1234567890",
        )
        assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.create(
            connection_id="1234567890",
            expires_at="2018-02-02T22:25:27.521Z",
            name="My-new-credential",
            tag="some_tag",
        )
        assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.telephony_credentials.with_raw_response.create(
            connection_id="1234567890",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = await response.parse()
        assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.telephony_credentials.with_streaming_response.create(
            connection_id="1234567890",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = await response.parse()
            assert_matches_type(TelephonyCredentialCreateResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.retrieve(
            "id",
        )
        assert_matches_type(TelephonyCredentialRetrieveResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.telephony_credentials.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = await response.parse()
        assert_matches_type(TelephonyCredentialRetrieveResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.telephony_credentials.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = await response.parse()
            assert_matches_type(TelephonyCredentialRetrieveResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.telephony_credentials.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.update(
            id="id",
        )
        assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.update(
            id="id",
            connection_id="987654321",
            expires_at="2018-02-02T22:25:27.521Z",
            name="My-new-updated-credential",
            tag="some_tag",
        )
        assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.telephony_credentials.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = await response.parse()
        assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.telephony_credentials.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = await response.parse()
            assert_matches_type(TelephonyCredentialUpdateResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.telephony_credentials.with_raw_response.update(
                id="",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.list()
        assert_matches_type(AsyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.list(
            filter={
                "name": "name",
                "resource_id": "resource_id",
                "sip_username": "sip_username",
                "status": "status",
                "tag": "tag",
            },
            page_number=0,
            page_size=0,
        )
        assert_matches_type(AsyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.telephony_credentials.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.telephony_credentials.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[TelephonyCredential], telephony_credential, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.delete(
            "id",
        )
        assert_matches_type(TelephonyCredentialDeleteResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.telephony_credentials.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = await response.parse()
        assert_matches_type(TelephonyCredentialDeleteResponse, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.telephony_credentials.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = await response.parse()
            assert_matches_type(TelephonyCredentialDeleteResponse, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.telephony_credentials.with_raw_response.delete(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_token(self, async_client: AsyncTelnyx) -> None:
        telephony_credential = await async_client.telephony_credentials.create_token(
            "id",
        )
        assert_matches_type(str, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create_token(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.telephony_credentials.with_raw_response.create_token(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        telephony_credential = await response.parse()
        assert_matches_type(str, telephony_credential, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create_token(self, async_client: AsyncTelnyx) -> None:
        async with async_client.telephony_credentials.with_streaming_response.create_token(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            telephony_credential = await response.parse()
            assert_matches_type(str, telephony_credential, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create_token(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.telephony_credentials.with_raw_response.create_token(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "AccessIPRangeListParams",
    "Filter",
    "FilterCidrBlock",
    "FilterCidrBlockCidrBlockPatternFilter",
    "FilterCreatedAt",
    "FilterCreatedAtDateRangeFilter",
]


class AccessIPRangeListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[cidr_block], filter[cidr_block][startswith],
    filter[cidr_block][endswith], filter[cidr_block][contains], filter[created_at].
    Supports complex bracket operations for dynamic filtering.
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterCidrBlockCidrBlockPatternFilter(TypedDict, total=False):
    """CIDR block pattern matching operations"""

    contains: str
    """Filter CIDR blocks containing the specified string"""

    endswith: str
    """Filter CIDR blocks ending with the specified string"""

    startswith: str
    """Filter CIDR blocks starting with the specified string"""


FilterCidrBlock: TypeAlias = Union[str, FilterCidrBlockCidrBlockPatternFilter]


class FilterCreatedAtDateRangeFilter(TypedDict, total=False):
    """Date range filtering operations"""

    gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time greater than"""

    gte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time greater than or equal to"""

    lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time less than"""

    lte: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for creation date-time less than or equal to"""


FilterCreatedAt: TypeAlias = Union[Union[str, datetime], FilterCreatedAtDateRangeFilter]


class FilterTyped(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[cidr_block], filter[cidr_block][startswith], filter[cidr_block][endswith], filter[cidr_block][contains], filter[created_at]. Supports complex bracket operations for dynamic filtering.
    """

    cidr_block: FilterCidrBlock
    """Filter by exact CIDR block match"""

    created_at: Annotated[FilterCreatedAt, PropertyInfo(format="iso8601")]
    """Filter by exact creation date-time"""


Filter: TypeAlias = Union[FilterTyped, Dict[str, object]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from .record import Record

__all__ = ["GlobalIPListResponse"]


class GlobalIPListResponse(Record):
    description: Optional[str] = None
    """A user specified description for the address."""

    ip_address: Optional[str] = None
    """The Global IP address."""

    name: Optional[str] = None
    """A user specified name for the address."""

    ports: Optional[Dict[str, object]] = None
    """A Global IP ports grouped by protocol code."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PortingLoaConfiguration", "Address", "Contact", "Logo"]


class Address(BaseModel):
    """The address of the company."""

    city: Optional[str] = None
    """The locality of the company"""

    country_code: Optional[str] = None
    """The country code of the company"""

    extended_address: Optional[str] = None
    """The extended address of the company"""

    state: Optional[str] = None
    """The administrative area of the company"""

    street_address: Optional[str] = None
    """The street address of the company"""

    zip_code: Optional[str] = None
    """The postal code of the company"""


class Contact(BaseModel):
    """The contact information of the company."""

    email: Optional[str] = None
    """The email address of the contact"""

    phone_number: Optional[str] = None
    """The phone number of the contact"""


class Logo(BaseModel):
    """The logo to be used in the LOA."""

    content_type: Optional[Literal["image/png"]] = None
    """The content type of the logo."""

    document_id: Optional[str] = None
    """Identifies the document that contains the logo."""


class PortingLoaConfiguration(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the LOA configuration."""

    address: Optional[Address] = None
    """The address of the company."""

    company_name: Optional[str] = None
    """The name of the company"""

    contact: Optional[Contact] = None
    """The contact information of the company."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    logo: Optional[Logo] = None
    """The logo to be used in the LOA."""

    name: Optional[str] = None
    """The name of the LOA configuration"""

    organization_id: Optional[str] = None
    """The organization that owns the LOA configuration"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["CallSiprecJsonParams"]


class CallSiprecJsonParams(TypedDict, total=False):
    account_sid: Required[str]

    connector_name: Annotated[str, PropertyInfo(alias="ConnectorName")]
    """The name of the connector to use for the SIPREC session."""

    include_metadata_custom_headers: Annotated[Literal[True, False], PropertyInfo(alias="IncludeMetadataCustomHeaders")]
    """
    When set, custom parameters will be added as metadata
    (recording.session.ExtensionParameters). Otherwise, they’ll be added to sip
    headers.
    """

    name: Annotated[str, PropertyInfo(alias="Name")]
    """Name of the SIPREC session.

    May be used to stop the SIPREC session from TeXML instruction.
    """

    secure: Annotated[Literal[True, False], PropertyInfo(alias="Secure")]
    """Controls whether to encrypt media sent to your SRS using SRTP and TLS.

    When set you need to configure SRS port in your connector to 5061.
    """

    session_timeout_secs: Annotated[int, PropertyInfo(alias="SessionTimeoutSecs")]
    """Sets `Session-Expires` header to the INVITE.

    A reinvite is sent every half the value set. Usefull for session keep alive.
    Minimum value is 90, set to 0 to disable.
    """

    sip_transport: Annotated[Literal["udp", "tcp", "tls"], PropertyInfo(alias="SipTransport")]
    """Specifies SIP transport protocol."""

    status_callback: Annotated[str, PropertyInfo(alias="StatusCallback")]
    """
    URL destination for Telnyx to send status callback events to for the siprec
    session.
    """

    status_callback_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="StatusCallbackMethod")]
    """HTTP request type used for `StatusCallback`."""

    track: Annotated[Literal["both_tracks", "inbound_track", "outbound_track"], PropertyInfo(alias="Track")]
    """The track to be used for siprec session.

    Can be `both_tracks`, `inbound_track` or `outbound_track`. Defaults to
    `both_tracks`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["IntegrationSecret"]


class IntegrationSecret(BaseModel):
    id: str

    created_at: datetime

    identifier: str

    record_type: str

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import FileTypes
from ..._utils import PropertyInfo

__all__ = ["AudioTranscribeParams"]


class AudioTranscribeParams(TypedDict, total=False):
    model: Required[Literal["distil-whisper/distil-large-v2", "openai/whisper-large-v3-turbo", "deepgram/nova-3"]]
    """ID of the model to use.

    `distil-whisper/distil-large-v2` is lower latency but English-only.
    `openai/whisper-large-v3-turbo` is multi-lingual but slightly higher latency.
    `deepgram/nova-3` supports English variants (en, en-US, en-GB, en-AU, en-NZ,
    en-IN) and only accepts mp3/wav files.
    """

    file: FileTypes
    """
    The audio file object to transcribe, in one of these formats: flac, mp3, mp4,
    mpeg, mpga, m4a, ogg, wav, or webm. File uploads are limited to 100 MB. Cannot
    be used together with `file_url`. Note: `deepgram/nova-3` only supports mp3 and
    wav formats.
    """

    file_url: str
    """
    Link to audio file in one of these formats: flac, mp3, mp4, mpeg, mpga, m4a,
    ogg, wav, or webm. Support for hosted files is limited to 100MB. Cannot be used
    together with `file`. Note: `deepgram/nova-3` only supports mp3 and wav formats.
    """

    language: str
    """The language of the audio to be transcribed.

    For `deepgram/nova-3`, only English variants are supported: `en`, `en-US`,
    `en-GB`, `en-AU`, `en-NZ`, `en-IN`. For `openai/whisper-large-v3-turbo`,
    supports multiple languages. `distil-whisper/distil-large-v2` does not support
    language parameter.
    """

    model_config: Dict[str, object]
    """Additional model-specific configuration parameters.

    Only allowed with `deepgram/nova-3` model. Can include Deepgram-specific options
    such as `smart_format`, `punctuate`, `diarize`, `utterance`, `numerals`, and
    `language`. If `language` is provided both as a top-level parameter and in
    `model_config`, the top-level parameter takes precedence.
    """

    response_format: Literal["json", "verbose_json"]
    """The format of the transcript output.

    Use `verbose_json` to take advantage of timestamps.
    """

    timestamp_granularities: Annotated[Literal["segment"], PropertyInfo(alias="timestamp_granularities[]")]
    """The timestamp granularities to populate for this transcription.

    `response_format` must be set verbose_json to use timestamp granularities.
    Currently `segment` is supported.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["MessagingProfileRetrieveMetricsParams"]


class MessagingProfileRetrieveMetricsParams(TypedDict, total=False):
    time_frame: Literal["1h", "3h", "24h", "3d", "7d", "30d"]
    """The time frame for metrics."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .wdr_report import WdrReport as WdrReport
from .detail_records_report_list_params import DetailRecordsReportListParams as DetailRecordsReportListParams
from .detail_records_report_create_params import DetailRecordsReportCreateParams as DetailRecordsReportCreateParams
from .detail_records_report_list_response import DetailRecordsReportListResponse as DetailRecordsReportListResponse
from .detail_records_report_create_response import (
    DetailRecordsReportCreateResponse as DetailRecordsReportCreateResponse,
)
from .detail_records_report_delete_response import (
    DetailRecordsReportDeleteResponse as DetailRecordsReportDeleteResponse,
)
from .detail_records_report_retrieve_response import (
    DetailRecordsReportRetrieveResponse as DetailRecordsReportRetrieveResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["TextToSpeechStreamParams"]


class TextToSpeechStreamParams(TypedDict, total=False):
    audio_format: Literal["pcm", "wav"]
    """Audio output format override.

    Supported for Telnyx `Natural`/`NaturalHD` models only. Accepted values: `pcm`,
    `wav`.
    """

    disable_cache: bool
    """When `true`, bypass the audio cache and generate fresh audio."""

    model_id: str
    """Model identifier for the chosen provider.

    Examples: `Natural`, `NaturalHD` (Telnyx); `Polly.Generative` (AWS).
    """

    provider: Literal["aws", "telnyx", "azure", "elevenlabs", "minimax", "murfai", "rime", "resemble"]
    """TTS provider.

    Defaults to `telnyx` if not specified. Ignored when `voice` is provided.
    """

    socket_id: str
    """Client-provided socket identifier for tracking.

    If not provided, one is generated server-side.
    """

    voice: str
    """
    Voice identifier in the format `provider.model_id.voice_id` or
    `provider.voice_id` (e.g. `telnyx.NaturalHD.Telnyx_Alloy` or
    `azure.en-US-AvaMultilingualNeural`). When provided, the `provider`, `model_id`,
    and `voice_id` are extracted automatically. Takes precedence over individual
    `provider`/`model_id`/`voice_id` parameters.
    """

    voice_id: str
    """Voice identifier for the chosen provider."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .alphanumeric_sender_id import AlphanumericSenderID

__all__ = ["AlphanumericSenderIDDeleteResponse"]


class AlphanumericSenderIDDeleteResponse(BaseModel):
    data: Optional[AlphanumericSenderID] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .sim_card_actions_summary import SimCardActionsSummary

__all__ = ["BulkSimCardActionRetrieveResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    action_type: Optional[Literal["bulk_set_public_ips"]] = None
    """The operation type. It can be one of the following: <br/>

    <ul>
    <li><code>bulk_set_public_ips</code> - set a public IP for each specified SIM card.</li>
    </ul>
    """

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    record_type: Optional[str] = None

    settings: Optional[Dict[str, object]] = None
    """A JSON object representation of the bulk action payload."""

    sim_card_actions_summary: Optional[List[SimCardActionsSummary]] = None

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""


class BulkSimCardActionRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .openai import (
    OpenAIResource,
    AsyncOpenAIResource,
    OpenAIResourceWithRawResponse,
    AsyncOpenAIResourceWithRawResponse,
    OpenAIResourceWithStreamingResponse,
    AsyncOpenAIResourceWithStreamingResponse,
)
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)

__all__ = [
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
    "OpenAIResource",
    "AsyncOpenAIResource",
    "OpenAIResourceWithRawResponse",
    "AsyncOpenAIResourceWithRawResponse",
    "OpenAIResourceWithStreamingResponse",
    "AsyncOpenAIResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["UsageGetAPIUsageResponse", "Data", "DataCategory", "DataTotal"]


class DataCategory(BaseModel):
    bytes_received: Optional[int] = None
    """The number of bytes received"""

    bytes_sent: Optional[int] = None
    """The number of bytes sent"""

    category: Optional[
        Literal[
            "list_bucket",
            "list_buckets",
            "get-bucket_location",
            "create_bucket",
            "stat_bucket",
            "get_bucket_versioning",
            "set_bucket_versioning",
            "get_obj",
            "put_obj",
            "delete_obj",
        ]
    ] = None
    """The category of the bucket operation"""

    ops: Optional[int] = None
    """The number of operations"""

    successful_ops: Optional[int] = None
    """The number of successful operations"""


class DataTotal(BaseModel):
    bytes_received: Optional[int] = None
    """The number of bytes received"""

    bytes_sent: Optional[int] = None
    """The number of bytes sent"""

    ops: Optional[int] = None
    """The number of operations"""

    successful_ops: Optional[int] = None
    """The number of successful operations"""


class Data(BaseModel):
    categories: Optional[List[DataCategory]] = None

    timestamp: Optional[datetime] = None
    """The time the usage was recorded"""

    total: Optional[DataTotal] = None


class UsageGetAPIUsageResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .rcs import (
    RcsResource,
    AsyncRcsResource,
    RcsResourceWithRawResponse,
    AsyncRcsResourceWithRawResponse,
    RcsResourceWithStreamingResponse,
    AsyncRcsResourceWithStreamingResponse,
)
from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)

__all__ = [
    "RcsResource",
    "AsyncRcsResource",
    "RcsResourceWithRawResponse",
    "AsyncRcsResourceWithRawResponse",
    "RcsResourceWithStreamingResponse",
    "AsyncRcsResourceWithStreamingResponse",
    "MessagingResource",
    "AsyncMessagingResource",
    "MessagingResourceWithRawResponse",
    "AsyncMessagingResourceWithRawResponse",
    "MessagingResourceWithStreamingResponse",
    "AsyncMessagingResourceWithStreamingResponse",
]
